LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.numeric_std.all;
entity ctest is
end ctest;
ARCHITECTURE camcontrol_a OF ctest IS
  signal trn_reset_n : std_logic;
  signal trn_clk : std_logic;
  
  component camera_model is
    Port ( DATA : out  STD_LOGIC_VECTOR (9 downto 0);
           CLK : in  STD_LOGIC;
           RESET_N : in  STD_LOGIC;
           LINE_VALID : out  STD_LOGIC;
           FRAME_VALID : out  STD_LOGIC;
           SPI_ENABLE : in  STD_LOGIC;
           SPI_CLK : in  STD_LOGIC;
           SPI_DATA : in  STD_LOGIC
           );
  end component;
  
  component blk_mem_gen_2_64
	port (
	clka: IN std_logic;
	wea: IN std_logic_VECTOR(0 downto 0);
	addra: IN std_logic_VECTOR(20 downto 0);
	dina: IN std_logic_VECTOR(1 downto 0);
	clkb: IN std_logic;
	addrb: IN std_logic_VECTOR(15 downto 0);
	doutb: OUT std_logic_VECTOR(63 downto 0));
end component;

component clk_wiz_v1_4
port
 (-- Clock in ports
  CLK_IN1           : in     std_logic;
  -- Clock out ports
  CLK_OUT1          : out    std_logic;
  CLK_OUT2          : out    std_logic
 );
end component;

component cameracontrol
  Port (
    DATA : in STD_LOGIC_VECTOR (9 downto 0);
    CAMERA_CLK : out STD_LOGIC;
    RESET_N : out STD_LOGIC;
    LINE_VALID : in STD_LOGIC;
    FRAME_VALID : in STD_LOGIC;
    SPI_ENABLE : out STD_LOGIC;
    SPI_CLK : out STD_LOGIC;
    SPI_DATA : out STD_LOGIC;
   
    clk_80mhz : in std_logic;
    clk_5x80mhz : in std_logic;
    reset_in : in std_logic;
    mem_write_en : out std_logic;
    mem_write_addr : out std_logic_vector(20 downto 0);
    mem_write_d : out std_logic_vector (1 downto 0)
  );
end component;

signal mastermodule_dma_rdaddr : std_logic_vector(31 downto 0);
signal mastermodule_dma_rddata : std_logic_vector(63 downto 0);
signal clk_80mhz : std_logic;
signal clk_5x80mhz : std_logic;
signal mem_write_en : std_logic;
signal mem_write_addr : std_logic_vector(20 downto 0);
signal mem_write_d : std_logic_vector (1 downto 0);
  
  signal CAM0_DATA : STD_LOGIC_VECTOR (9 downto 0);
  signal CAM0_CLK : STD_LOGIC;
  signal CAM0_RESET_N : STD_LOGIC;
  signal CAM0_LINE_VALID : STD_LOGIC;
  signal CAM0_FRAME_VALID : STD_LOGIC;
--  signal CAM0_SPI_ENABLE : STD_LOGIC;
--  signal CAM0_SPI_CLK : STD_LOGIC;
--  signal CAM0_SPI_DATA : STD_LOGIC;
begin
mastermodule_dma_rdaddr <= (others=>'0');

reset_proc: process
begin
  trn_reset_n <= '0';
  wait for 32 ns;
  trn_reset_n <= '1';
  wait for 100000000 ns;
end process;

clk_proc: process
begin   
  loop
    trn_clk <= '1';
    wait for 2 ns;
    trn_clk <= '0';
    wait for 2 ns;          
  end loop;     
end process;


  camera : camera_model
  port map(
    DATA => CAM0_DATA,
    CLK => CAM0_CLK,
    RESET_N => CAM0_RESET_N,
    LINE_VALID => CAM0_LINE_VALID,
    FRAME_VALID => CAM0_FRAME_VALID,
    SPI_ENABLE => '0',
    SPI_CLK => '0',
    SPI_DATA => '0'
  );

buffer_mem : blk_mem_gen_2_64
port map (
  clka => clk_5x80mhz,
  wea(0) => mem_write_en,
  addra => mem_write_addr,
  dina => mem_write_d,
  clkb => trn_clk,
  addrb => mastermodule_dma_rdaddr(15 downto 0),
  doutb => mastermodule_dma_rddata
);

clk_gen : clk_wiz_v1_4
port map (
  CLK_IN1 => trn_clk,
  CLK_OUT1 => clk_80mhz,
  CLK_OUT2 => clk_5x80mhz
);

cam0 : cameracontrol
port map (
  DATA => CAM0_DATA,
  CAMERA_CLK => CAM0_CLK,
  RESET_N => CAM0_RESET_N,
  LINE_VALID => CAM0_LINE_VALID,
  FRAME_VALID => CAM0_FRAME_VALID,
  SPI_ENABLE => open,
  SPI_CLK => open,
  SPI_DATA => open,
  
  clk_80mhz => clk_80mhz,
  clk_5x80mhz => clk_5x80mhz,
  reset_in => trn_reset_n,
  mem_write_en => mem_write_en,
  mem_write_addr => mem_write_addr,
  mem_write_d => mem_write_d
);

  
end;