----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:42:40 05/18/2010 
-- Design Name: 
-- Module Name:    camera_model - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity camera_model is
generic (
  --default parameters for operation at 640x480, 80 MHz
  picture_width : integer := 640;
  picture_height : integer := 480;
  --number of cycles between the end of a line and the beginning of the next
  line_overhead : integer := 32;
  --number of cycles between the end of a frame and the beginning of the next
  frame_overhead : integer := 624
);
Port (
  DATA : out  STD_LOGIC_VECTOR (9 downto 0);
  CLK : in  STD_LOGIC;
  RESET_N : in  STD_LOGIC;
  LINE_VALID : out  STD_LOGIC;
  FRAME_VALID : out  STD_LOGIC;
  SPI_ENABLE : in  STD_LOGIC;
  SPI_CLK : in  STD_LOGIC;
  SPI_DATA : in  STD_LOGIC
);
end camera_model;

architecture Behavioral of camera_model is
--  signal CLK : std_logic;
--  signal RESET_N : std_logic;
begin

--Processes for standalone simulation
--clk_proc: process
--begin   
--  loop
--    CLK <= '1';
--    wait for 12.5 ns;
--    CLK <= '0';
--    wait for 12.5 ns;          
--  end loop;     
--end process;

--reset_proc: process
--begin
--  RESET_N <= '0';
--  wait for 20 ns;
--  RESET_N <= '1';
--  wait for 100000000 ns;
--end process;

process (CLK, RESET_N)
  variable linecnt : integer range 0 to picture_height;
  variable pixelcnt : integer range 0 to picture_width;
  variable line_waitcnt : integer range 0 to line_overhead;
  variable frame_waitcnt : integer range 0 to frame_overhead;
  variable data_int : unsigned(9 downto 0);
begin
  DATA <= std_logic_vector(data_int);
  if RESET_N = '0' then
    linecnt := 0;
    pixelcnt := 0;
    line_waitcnt := 0;
    frame_waitcnt := 0;
    data_int := "1111111111";
    LINE_VALID <= '0';
    FRAME_VALID <= '0';
  elsif rising_edge(CLK) then
    if frame_waitcnt < frame_overhead then
      frame_waitcnt := frame_waitcnt + 1;
    elsif line_waitcnt < line_overhead then
      line_waitcnt := line_waitcnt + 1;
    else
      if pixelcnt >= picture_width then
        pixelcnt := 0;
        line_waitcnt := 0;
        linecnt := linecnt + 1;
        LINE_VALID <= '0';
        if linecnt >= picture_height then
          linecnt := 0;
          frame_waitcnt := 0;
          FRAME_VALID <= '0';
        end if;
      else
        pixelcnt := pixelcnt + 1;
        data_int := data_int + 1;
        LINE_VALID <= '1';
        FRAME_VALID <= '1';
      end if;
    end if;
  end if;
end process;

end Behavioral;

