
library ieee,unisim;
use ieee.std_logic_1164.all;
use unisim.VCOMPONENTS.all;

entity top_module is
	generic 
		(
		SIM_MODE			: boolean:=FALSE
		);
	port
		(
		-- General signals

	--Add for protection
prot0_out : out std_logic;	
prot2_in 	: in std_logic_vector(1 downto 0);
prot2_out : out std_logic;   

        refclk_p        	: in    std_logic;
        refclk_n        	: in    std_logic;
		mperst				: in 	std_logic;
        rxp             	: in    std_logic_vector (3 downto 0);
        rxn             	: in    std_logic_vector (3 downto 0);
        txp             	: out 	std_logic_vector (3 downto 0);
        txn             	: out 	std_logic_vector (3 downto 0);
        
		-- User Leds and switches
		usr_sw				: in  	std_logic_vector (3 downto 0);
		usr_led 			: out 	std_logic_vector (7 downto 0)
    
  --Camera signals
  --;
  --CAM0_DATA : in STD_LOGIC_VECTOR (9 downto 0);
  --CAM0_CLK : in STD_LOGIC;
  --CAM0_RESET_N : out STD_LOGIC;
  --CAM0_LINE_VALID : in STD_LOGIC;
  --CAM0_FRAME_VALID : in STD_LOGIC;
  --CAM0_SPI_ENABLE : out STD_LOGIC;
  --CAM0_SPI_CLK : out STD_LOGIC;
  --CAM0_SPI_DATA : out STD_LOGIC
		);
end top_module;

architecture top_module_arch of top_module is
  --parameters for camera model instances.
  constant picture_width : integer := 640;
  constant picture_height : integer := 480;
  constant line_overhead : integer := 32;
  constant frame_overhead : integer := 624;

	component v6_pcie_v1_3
		generic (
		PL_FAST_TRAIN : boolean :=FALSE
		);
		port (
		pci_exp_txp : out std_logic_vector (3 downto 0);
		pci_exp_txn : out std_logic_vector (3 downto 0);
		pci_exp_rxp : in std_logic_vector (3 downto 0);
		pci_exp_rxn : in std_logic_vector (3 downto 0);
		trn_clk : out std_logic;
		trn_reset_n : out std_logic;
		trn_lnk_up_n : out std_logic;                
		trn_tbuf_av : out std_logic_vector(5 downto 0);
		trn_tcfg_req_n : out std_logic;
		trn_tcfg_gnt_n : in std_logic;
		trn_terr_drop_n : out std_logic;		
		trn_td : in std_logic_vector(63 downto 0);
		trn_trem_n : in std_logic;
		trn_tsof_n : in std_logic;
		trn_teof_n : in std_logic;
		trn_tsrc_rdy_n : in std_logic;
		trn_tdst_rdy_n : out std_logic;
		trn_tsrc_dsc_n : in std_logic;
		trn_terrfwd_n : in std_logic; 
		trn_tstr_n : in std_logic;
		trn_rd : out std_logic_vector(63 downto 0);
		trn_rrem_n : out std_logic;
		trn_rsof_n : out std_logic;
		trn_reof_n : out std_logic;
		trn_rsrc_rdy_n : out std_logic;
		trn_rsrc_dsc_n : out std_logic;
		trn_rdst_rdy_n : in std_logic;
		trn_rerrfwd_n : out std_logic;
		trn_rnp_ok_n : in std_logic;
		trn_rbar_hit_n : out std_logic_vector(6 downto 0); 		
		trn_fc_cpld : out std_logic_vector(11 downto 0);
		trn_fc_cplh : out std_logic_vector(7 downto 0);
		trn_fc_npd : out std_logic_vector(11 downto 0);
		trn_fc_nph : out std_logic_vector(7 downto 0); 
		trn_fc_pd : out std_logic_vector(11 downto 0);
		trn_fc_ph : out std_logic_vector(7 downto 0); 
		trn_fc_sel : in std_logic_vector(2 downto 0);						
		cfg_do : out std_logic_vector(31 downto 0);
		cfg_rd_wr_done_n : out std_logic;
		cfg_di : in std_logic_vector(31 downto 0);
		cfg_byte_en_n : in std_logic_vector(3 downto 0);
		cfg_dwaddr : in std_logic_vector(9 downto 0);
		cfg_wr_en_n : in std_logic;
		cfg_rd_en_n : in std_logic;		
		cfg_err_cor_n : in std_logic;
		cfg_err_ur_n : in std_logic;
		cfg_err_ecrc_n : in std_logic;
		cfg_err_cpl_timeout_n : in std_logic;
		cfg_err_cpl_abort_n : in std_logic;
		cfg_err_cpl_unexpect_n : in std_logic;
		cfg_err_posted_n : in std_logic;
		cfg_err_locked_n : in std_logic;
		cfg_err_tlp_cpl_header : in std_logic_vector(47 downto 0);
		cfg_err_cpl_rdy_n : out std_logic;
		cfg_interrupt_n : in std_logic;
		cfg_interrupt_rdy_n : out std_logic;
		cfg_interrupt_assert_n : in std_logic;
		cfg_interrupt_di : in std_logic_vector(7 downto 0);
		cfg_interrupt_do : out std_logic_vector(7 downto 0);
		cfg_interrupt_mmenable : out std_logic_vector(2 downto 0);
		cfg_interrupt_msienable : out std_logic;      
		cfg_interrupt_msixenable : out std_logic;    
		cfg_interrupt_msixfm : out std_logic;    				
		cfg_to_turnoff_n : out std_logic;
  		cfg_turnoff_ok_n : in std_logic;		
		cfg_pm_wake_n : in std_logic;		
		cfg_pcie_link_state_n : out std_logic_vector(2 downto 0);
		cfg_trn_pending_n : in std_logic;
		cfg_bus_number : out std_logic_vector(7 downto 0);
		cfg_device_number : out std_logic_vector(4 downto 0);
		cfg_function_number : out std_logic_vector(2 downto 0);
		cfg_dsn : in std_logic_vector(63 downto 0);
		cfg_status : out std_logic_vector(15 downto 0);
		cfg_command : out std_logic_vector(15 downto 0);
		cfg_dstatus : out std_logic_vector(15 downto 0);
		cfg_dcommand : out std_logic_vector(15 downto 0);
		cfg_lstatus : out std_logic_vector(15 downto 0);
		cfg_lcommand : out std_logic_vector(15 downto 0);
		cfg_dcommand2 : out std_logic_vector(15 downto 0);		
		cfg_pmcsr_pme_en : out std_logic;                                       
		cfg_pmcsr_pme_status : out std_logic;                                       
		cfg_pmcsr_powerstate : out std_logic_vector(1 downto 0);                                 
--		lnk_clk_en : out std_logic;                                        		                                
		pl_initial_link_width : out std_logic_vector(2 downto 0);                                 
		pl_lane_reversal_mode : out std_logic_vector(1 downto 0);                                   
		pl_link_gen2_capable : out std_logic;                                        
		pl_link_partner_gen2_supported : out std_logic;                                       
		pl_link_upcfg_capable : out std_logic;                                       
		pl_ltssm_state : out std_logic_vector(5 downto 0);                                   
		pl_received_hot_rst : out std_logic;                                        
		pl_sel_link_rate : out std_logic;                                        
		pl_sel_link_width : out std_logic_vector(1 downto 0);                                   
		pl_directed_link_auton : in std_logic;                                         
		pl_directed_link_change : in std_logic_vector(1 downto 0);                                   
		pl_directed_link_speed : in std_logic;                                         
		pl_directed_link_width : in std_logic_vector(1 downto 0);                                   
		pl_upstream_prefer_deemph : in std_logic; 
--		received_hot_reset : out std_logic;		                                           		
		sys_clk : in std_logic;
		sys_reset_n : in std_logic);
	end component;
      
	component ezdma is
	  port (
	    clk : in std_logic;
	    rstn : in std_logic;
	    test_mode : in std_logic_vector(15 downto 0);
	    trn_lnk_up_n : in std_logic;
	    trn_td : out std_logic_vector(63 downto 0);
	    trn_trem_n : out std_logic_vector(1 downto 0);
	    trn_tsof_n : out std_logic;
	    trn_teof_n : out std_logic;
	    trn_tsrc_dsc_n : out std_logic;
	    trn_tsrc_rdy_n : out std_logic;
	    trn_tdst_rdy_n : in std_logic;
	    trn_terr_drop_n : in std_logic;
	    trn_terrfwd_n : out std_logic;
	    trn_tstr_n : out std_logic;
	    trn_tbuf_av : in std_logic_vector(5 downto 0);
	    trn_rd : in std_logic_vector(63 downto 0);
--	    trn_rrem_n : in std_logic_vector(1 downto 0);
	    trn_rsof_n : in std_logic;
	    trn_reof_n : in std_logic;
	    trn_rsrc_dsc_n : in std_logic;
	    trn_rsrc_rdy_n : in std_logic;
	    trn_rbar_hit_n : in std_logic_vector(6 downto 0);
	    trn_rdst_rdy_n : out std_logic;
	    trn_rerrfwd_n : in std_logic;
	    trn_rnp_ok_n : out std_logic;
	    cfg_err_cor_n : out std_logic;
	    cfg_err_cpl_abort_n : out std_logic;
	    cfg_err_cpl_timeout_n : out std_logic;
	    cfg_err_cpl_unexpect_n : out std_logic;
	    cfg_err_ecrc_n : out std_logic;
	    cfg_err_posted_n : out std_logic;
	    cfg_err_tlp_cpl_header : out std_logic_vector(47 downto 0);
	    cfg_err_ur_n : out std_logic;
	    cfg_err_cpl_rdy_n : in std_logic;
	    cfg_err_locked_n : out std_logic;
	    cfg_interrupt_n : out std_logic;
	    cfg_interrupt_rdy_n : in std_logic;
	    cfg_pcie_link_state_n : in std_logic_vector(2 downto 0);
	    cfg_interrupt_assert_n : out std_logic;
	    cfg_interrupt_di : out std_logic_vector(7 downto 0);
	    cfg_interrupt_do : in std_logic_vector(7 downto 0);
	    cfg_interrupt_mmenable : in std_logic_vector(2 downto 0);
	    cfg_interrupt_msienable : in std_logic;
	    cfg_trn_pending_n : out std_logic;
	    cfg_bus_number : in std_logic_vector(7 downto 0);
	    cfg_device_number : in std_logic_vector(4 downto 0);
	    cfg_function_number : in std_logic_vector(2 downto 0);
	    cfg_status : in std_logic_vector(15 downto 0);
	    cfg_command : in std_logic_vector(15 downto 0);
	    cfg_dstatus : in std_logic_vector(15 downto 0);
	    cfg_dcommand : in std_logic_vector(15 downto 0);
	    cfg_lstatus : in std_logic_vector(15 downto 0);
	    cfg_lcommand : in std_logic_vector(15 downto 0);
	    int_request : in std_logic;
	    int_ack : out std_logic;
	    int_msgnum : in std_logic_vector(4 downto 0);
	    cfg_prmcsr : out std_logic_vector(31 downto 0);
	    cfg_devcsr : out std_logic_vector(31 downto 0);
	    cfg_linkcsr : out std_logic_vector(31 downto 0);
	    cfg_msicsr : out std_logic_vector(15 downto 0);
	    cfg_ltssm : out std_logic_vector(4 downto 0);
	    slv_dataout : out std_logic_vector(63 downto 0);
	    slv_bytevalid : out std_logic_vector(7 downto 0);
	    slv_bytecount : out std_logic_vector(12 downto 0);
	    slv_dwcount : out std_logic_vector(10 downto 0);
	    slv_addr : out std_logic_vector(63 downto 0);
	    slv_bar : out std_logic_vector(6 downto 0);
	    slv_readreq : out std_logic;
	    slv_cpladdr : out std_logic_vector(31 downto 0);
	    slv_cplparam : out std_logic_vector(7 downto 0);
	    slv_writereq : out std_logic;
	    slv_write : out std_logic;
	    slv_lastwrite : out std_logic;
	    slv_io : out std_logic;
	    slv_accept : in std_logic;
	    slv_abort : in std_logic;
	    slv_ur : in std_logic;
	    dma_rd : out std_logic;
	    dma_rdaddr : out std_logic_vector(15 downto 0);
	    dma_rdchannel : out std_logic_vector(15 downto 0);
	    dma_rddata : in std_logic_vector(63 downto 0);
	    dma_wr : out std_logic;
	    dma_wraddr : out std_logic_vector(15 downto 0);
	    dma_wrchannel : out std_logic_vector(15 downto 0);
	    dma_wrdata : out std_logic_vector(63 downto 0);
	    dma_wrbytevalid : out std_logic_vector(7 downto 0);
	    dma0_regin : in std_logic_vector(127 downto 0);
	    dma0_regout : out std_logic_vector(127 downto 0);
	    dma0_param : in std_logic_vector(23 downto 0);
	    dma0_control : in std_logic_vector(5 downto 0);
	    dma0_status : out std_logic_vector(3 downto 0);
	    dma0_fifocnt : in std_logic_vector(12 downto 0);

	--Add for protection
prot0_out : out std_logic;	
prot2_in 	: in std_logic_vector(1 downto 0);
prot2_out : out std_logic   

);
	end component;
    
	component pcie_app
	generic
		(
		DATAPATH       : integer := 64;
		CORE_FREQ		: integer:=125
		);
	port(
	    clk 				: in std_logic;
	    reset_n 				: in std_logic;

      int_request : out std_logic;
      int_ack : in std_logic;

   	  slv_dataout 		: in std_logic_vector((DATAPATH-1)  downto 0);
	    slv_bytevalid 		: in std_logic_vector((DATAPATH/8-1) downto 0);
	    slv_bytecount 		: in std_logic_vector(12 downto 0);
	    slv_addr 			: in std_logic_vector(63 downto 0);
	    slv_bar 			: in std_logic_vector(6 downto 0);
	    slv_readreq 		: in std_logic;
	    slv_writereq 		: in std_logic;
	    slv_write 			: in std_logic;
      slv_lastwrite   : in std_logic;
	    slv_accept 			: out std_logic;
	    slv_abort 			: out std_logic;
      slv_ur    			: out std_logic;


	    dma_rd 				: in std_logic;
	    dma_rdaddr 			: in std_logic_vector(15 downto 0);
	    dma_rdchannel 		: in std_logic_vector(15 downto 0);
	    dma_rddata 			: out std_logic_vector((DATAPATH-1) downto 0);
	    dma_wr 				: in std_logic;
	    dma_wrchannel 		: in std_logic_vector(15 downto 0);
	    dma_wrdata 			: in std_logic_vector((DATAPATH-1) downto 0);

	    dma0_regin 			: out std_logic_vector(127 downto 0);
	    dma0_regout 		: in std_logic_vector(127 downto 0);
	    dma0_param 			: out std_logic_vector(23 downto 0);
	    dma0_control 		: out std_logic_vector(5 downto 0);
	    dma0_status 		: in std_logic_vector(3 downto 0);
	    dma0_fifocnt 		: out std_logic_vector(12 downto 0);

  CAM0_DATA : in STD_LOGIC_VECTOR (9 downto 0);
  CAM0_CLK : out STD_LOGIC;
  CAM0_RESET_N : out STD_LOGIC;
  CAM0_LINE_VALID : in STD_LOGIC;
  CAM0_FRAME_VALID : in STD_LOGIC;
  CAM0_SPI_ENABLE : out STD_LOGIC;
  CAM0_SPI_CLK : out STD_LOGIC;
  CAM0_SPI_DATA : out STD_LOGIC

--   		cfg_linkcsr 		: in std_logic_vector(31 downto 0);
--   		cfg_ltssm			: in std_logic_vector (4 downto 0);
--	    usr_led				: out std_logic_vector(3 downto 0)
  );
	end component;

	---------------------------------------------------------------------------

	constant DATAPATH : integer:=64;

    -- Glue between Xilinx core and PCIEZ
	signal trn_lnk_up_n 			: std_logic;
	signal trn_td 					: std_logic_vector(DATAPATH-1 downto 0);
	signal trn_trem_n 				: std_logic_vector(1 downto 0);
	signal trn_rrem_n 				: std_logic_vector(1 downto 0);
	signal trn_tsof_n 				: std_logic;
	signal trn_teof_n 				: std_logic;
	signal trn_tstr_n 				: std_logic;
	signal trn_terr_drop_n 			: std_logic;
	signal trn_tsrc_dsc_n 			: std_logic;
	signal trn_tsrc_rdy_n 			: std_logic;
	signal trn_tdst_rdy_n 			: std_logic;
	signal trn_terrfwd_n 			: std_logic;
	signal trn_tbuf_av 				: std_logic_vector(5 downto 0);
	signal trn_rd 					: std_logic_vector(DATAPATH-1 downto 0);
	signal trn_rsof_n 				: std_logic;
	signal trn_reof_n 				: std_logic;
	signal trn_rsrc_dsc_n 			: std_logic;
	signal trn_rsrc_rdy_n 			: std_logic;
	signal trn_rbar_hit_n 			: std_logic_vector(6 downto 0);
	signal trn_rdst_rdy_n 			: std_logic;
	signal trn_rerrfwd_n 			: std_logic;
	signal trn_rnp_ok_n 			: std_logic;
	signal cfg_err_cor_n			: std_logic;
	signal cfg_err_ur_n				: std_logic;
	signal cfg_err_ecrc_n			: std_logic;
	signal cfg_err_cpl_timeout_n	: std_logic;
	signal cfg_err_cpl_abort_n		: std_logic;
	signal cfg_err_cpl_unexpect_n	: std_logic;
	signal cfg_err_posted_n			: std_logic;
	signal cfg_err_locked_n			: std_logic;
	signal cfg_err_tlp_cpl_header	: std_logic_vector(47 downto 0);
	signal cfg_err_cpl_rdy_n		: std_logic;
	signal cfg_interrupt_n			: std_logic;
	signal cfg_interrupt_rdy_n		: std_logic;
	signal cfg_interrupt_assert_n	: std_logic;
	signal cfg_interrupt_di			: std_logic_vector(7 downto 0);
	signal cfg_interrupt_do			: std_logic_vector(7 downto 0);
	signal cfg_interrupt_mmenable	: std_logic_vector(2 downto 0);
	signal cfg_interrupt_msienable	: std_logic;
	signal cfg_pcie_link_state_n	: std_logic_vector(2 downto 0);
	signal cfg_trn_pending_n		: std_logic;
	signal cfg_bus_number			: std_logic_vector(7 downto 0);
	signal cfg_device_number		: std_logic_vector(4 downto 0);
	signal cfg_function_number		: std_logic_vector(2 downto 0);
	signal cfg_status				: std_logic_vector(15 downto 0);
	signal cfg_command				: std_logic_vector(15 downto 0);
	signal cfg_dstatus				: std_logic_vector(15 downto 0);
	signal cfg_dcommand				: std_logic_vector(15 downto 0);
	signal cfg_lstatus				: std_logic_vector(15 downto 0);
	signal cfg_lcommand				: std_logic_vector(15 downto 0);

	-- Connection between EZ and reference design logic
	-- Slave interface
	signal slv_dataout 								: std_logic_vector(DATAPATH-1 downto 0);
	signal slv_bytevalid 							: std_logic_vector(DATAPATH/8-1 downto 0);
	signal slv_bytecount 							: std_logic_vector(12 downto 0);
	signal slv_addr 								: std_logic_vector(63 downto 0);
	signal slv_bar 									: std_logic_vector(6 downto 0);
	signal slv_cpladdr 								: std_logic_vector(31 downto 0);
	signal slv_cplparam 							: std_logic_vector(7 downto 0);
	signal slv_writereq,slv_readreq,slv_write,slv_lastwrite		: std_logic;
	signal slv_accept,slv_abort,slv_ur						: std_logic;
	signal int_request								: std_logic;
  signal int_ack : std_logic;

	-- DMA interface
	signal dma_wr,dma_rd 							: std_logic;
	signal dma_rdaddr 								: std_logic_vector(15 downto 0);
	signal dma_wrchannel,dma_rdchannel 				: std_logic_vector(15 downto 0);
	signal dma_wrdata,dma_rddata 					: std_logic_vector(DATAPATH-1 downto 0);

	signal dma0_regin,dma1_regin,dma2_regin			: std_logic_vector(127 downto 0);
	signal dma0_regout,dma1_regout					: std_logic_vector(127 downto 0);
	signal dma0_param,dma1_param,dma2_param			: std_logic_vector(23 downto 0);
	signal dma0_control,dma1_control,dma2_control	: std_logic_vector(5 downto 0);
	signal dma0_status,dma1_status,dma2_status		: std_logic_vector(3 downto 0);
	signal dma0_fifocnt,dma1_fifocnt,dma2_fifocnt	: std_logic_vector(12 downto 0);

	signal cfg_ltssm								: std_logic_vector (4 downto 0);
	signal cfg_linkcsr 								: std_logic_vector(31 downto 0);

	-- Clock & reset
	signal ZERO,ONE									: std_logic_vector (127 downto 0);
	signal trn_clk,trn_reset_n,refclk				: std_logic;   
	signal rst,rstn									: std_logic;  	

  component camera_model is
	generic (
	  picture_width : integer := 640;
	  picture_height : integer := 480;
	  line_overhead : integer := 32;
	  frame_overhead : integer := 624
	);
	Port (
	  DATA : out  STD_LOGIC_VECTOR (9 downto 0);
	  CLK : in  STD_LOGIC;
	  RESET_N : in  STD_LOGIC;
	  LINE_VALID : out  STD_LOGIC;
	  FRAME_VALID : out  STD_LOGIC;
	  SPI_ENABLE : in  STD_LOGIC;
	  SPI_CLK : in  STD_LOGIC;
	  SPI_DATA : in  STD_LOGIC
	);
  end component;

  signal CAM0_DATA : STD_LOGIC_VECTOR (9 downto 0);
  signal CAM0_CLK : STD_LOGIC;
  signal CAM0_RESET_N : STD_LOGIC;
  signal CAM0_LINE_VALID : STD_LOGIC;
  signal CAM0_FRAME_VALID : STD_LOGIC;
  signal CAM0_SPI_ENABLE : STD_LOGIC;
  signal CAM0_SPI_CLK : STD_LOGIC;
  signal CAM0_SPI_DATA : STD_LOGIC;

begin

	-- Glue
  	ZERO <=(others=>'0');
  	ONE <=(others=>'1');
	
	-------------------------------------------------
	-- Xilinx HardIP core
	-------------------------------------------------

    core : v6_pcie_v1_3
    generic map ( 
		PL_FAST_TRAIN => SIM_MODE
		)      
	port map (
		pci_exp_txp					=> txp,
		pci_exp_txn					=> txn,
		pci_exp_rxp					=> rxp,
		pci_exp_rxn					=> rxn,
		trn_clk						=> trn_clk,
		trn_reset_n					=> trn_reset_n,
		trn_lnk_up_n				=> trn_lnk_up_n,
		trn_td						=> trn_td,
		trn_trem_n					=> trn_trem_n(0),
		trn_tsof_n					=> trn_tsof_n,
		trn_teof_n					=> trn_teof_n,
		trn_tsrc_rdy_n				=> trn_tsrc_rdy_n,
		trn_tdst_rdy_n				=> trn_tdst_rdy_n,
		trn_tsrc_dsc_n				=> trn_tsrc_dsc_n,
		trn_terrfwd_n				=> trn_terrfwd_n,
		trn_tbuf_av					=> trn_tbuf_av,		
		trn_tcfg_req_n				=> open,
		trn_tcfg_gnt_n				=> ZERO(0),
		trn_terr_drop_n				=> trn_terr_drop_n,
		trn_tstr_n					=> trn_tstr_n,					
		trn_rd						=> trn_rd,
		trn_rrem_n			   		=> trn_rrem_n(0),
		trn_rsof_n					=> trn_rsof_n,
		trn_reof_n					=> trn_reof_n,
		trn_rsrc_rdy_n				=> trn_rsrc_rdy_n,
		trn_rsrc_dsc_n				=> trn_rsrc_dsc_n,
		trn_rdst_rdy_n				=> trn_rdst_rdy_n,
		trn_rerrfwd_n				=> trn_rerrfwd_n,
		trn_rnp_ok_n				=> trn_rnp_ok_n,
		trn_rbar_hit_n				=> trn_rbar_hit_n,
		trn_fc_cpld					=> open,
		trn_fc_cplh					=> open,
		trn_fc_npd				    => open,
		trn_fc_nph					=> open,
		trn_fc_pd				    => open,
		trn_fc_ph					=> open,
		trn_fc_sel					=> ZERO(2 downto 0),		
		cfg_do						=> open,
		cfg_rd_wr_done_n			=> open,
		cfg_di						=> ZERO(31 downto 0),
		cfg_byte_en_n				=> ZERO(3 downto 0),
		cfg_dwaddr					=> ZERO(9 downto 0),
		cfg_wr_en_n					=> ONE(0),
		cfg_rd_en_n					=> ONE(0),
		cfg_err_cor_n				=> cfg_err_cor_n,
		cfg_err_ur_n				=> cfg_err_ur_n,
		cfg_err_ecrc_n				=> cfg_err_ecrc_n,
		cfg_err_cpl_timeout_n		=> cfg_err_cpl_timeout_n,
		cfg_err_cpl_abort_n			=> cfg_err_cpl_abort_n,
		cfg_err_cpl_unexpect_n		=> cfg_err_cpl_unexpect_n,
		cfg_err_posted_n			=> cfg_err_posted_n,
		cfg_err_locked_n			=> cfg_err_locked_n,
		cfg_err_tlp_cpl_header		=> cfg_err_tlp_cpl_header,
		cfg_err_cpl_rdy_n			=> cfg_err_cpl_rdy_n,
		cfg_interrupt_n				=> cfg_interrupt_n,
		cfg_interrupt_rdy_n			=> cfg_interrupt_rdy_n,
		cfg_interrupt_assert_n		=> cfg_interrupt_assert_n,
		cfg_interrupt_di			=> cfg_interrupt_di,
		cfg_interrupt_do			=> cfg_interrupt_do,
		cfg_interrupt_mmenable		=> cfg_interrupt_mmenable,
		cfg_interrupt_msienable		=> cfg_interrupt_msienable,	
		cfg_interrupt_msixenable 	=> open,  
		cfg_interrupt_msixfm 		=> open,		
		cfg_to_turnoff_n			=> open,
		cfg_turnoff_ok_n			=> ONE(0),
		cfg_pm_wake_n				=> ONE(0),
		cfg_pcie_link_state_n		=> cfg_pcie_link_state_n,
		cfg_trn_pending_n			=> cfg_trn_pending_n,
		cfg_bus_number				=> cfg_bus_number,
		cfg_device_number			=> cfg_device_number,
		cfg_function_number			=> cfg_function_number,
		cfg_dsn			   			=> ZERO(63 downto 0),
		cfg_status					=> cfg_status,
		cfg_command					=> cfg_command,
		cfg_dstatus			   		=> cfg_dstatus,
		cfg_dcommand				=> cfg_dcommand,
		cfg_lstatus					=> cfg_lstatus,
		cfg_lcommand				=> cfg_lcommand,
		cfg_dcommand2				=> open, 
		cfg_pmcsr_pme_en			=> open, 
		cfg_pmcsr_pme_status		=> open, 
		cfg_pmcsr_powerstate		=> open,    
--		lnk_clk_en					=> open, 	
  		pl_initial_link_width 		=> open, 
  		pl_lane_reversal_mode 		=> open,
  		pl_link_gen2_capable 		=> open, 
  		pl_link_partner_gen2_supported =>open,
  		pl_link_upcfg_capable		=> open,
  		pl_ltssm_state 				=> open,
  		pl_received_hot_rst 		=> open,
  		pl_sel_link_rate 			=> open,
  		pl_sel_link_width 			=> open,
  		pl_directed_link_auton 		=> ZERO(0), 
  		pl_directed_link_change		=> ZERO(1 downto 0), 
  		pl_directed_link_speed		=> ZERO(0),  
  		pl_directed_link_width		=> ZERO(1 downto 0), 
  		pl_upstream_prefer_deemph	=> ZERO(0),
--  		received_hot_reset			=> open, 
		sys_clk						=> refclk,
		sys_reset_n					=> mperst
		);    
          	
	-------------------------------------------------
	-- EZ layer
	-------------------------------------------------

	ezdma_module : ezdma
	port map (
		clk							=> trn_clk,

	--Add for protection
	prot0_out => prot0_out,
	prot2_in => prot2_in,
	prot2_out => prot2_out,

		rstn						=> rstn,
		test_mode	    			=> ZERO(15 downto 0),

		trn_lnk_up_n				=> trn_lnk_up_n,
		trn_td						=> trn_td,
		trn_trem_n					=> trn_trem_n,
		trn_tsof_n					=> trn_tsof_n,
		trn_teof_n					=> trn_teof_n,
		trn_tsrc_rdy_n				=> trn_tsrc_rdy_n,
		trn_tdst_rdy_n				=> trn_tdst_rdy_n,
		trn_terr_drop_n				=> trn_terr_drop_n,
		trn_tsrc_dsc_n				=> trn_tsrc_dsc_n,
		trn_terrfwd_n				=> trn_terrfwd_n,
		trn_tbuf_av					=> trn_tbuf_av,
		trn_tstr_n					=> trn_tstr_n,
		trn_rd						=> trn_rd,
--		trn_rrem_n					=> trn_rrem_n,
		trn_rsof_n					=> trn_rsof_n,
		trn_reof_n					=> trn_reof_n,
		trn_rsrc_rdy_n				=> trn_rsrc_rdy_n,
		trn_rsrc_dsc_n				=> trn_rsrc_dsc_n,
		trn_rdst_rdy_n				=> trn_rdst_rdy_n,
		trn_rerrfwd_n				=> trn_rerrfwd_n,
		trn_rnp_ok_n				=> trn_rnp_ok_n,
		trn_rbar_hit_n				=> trn_rbar_hit_n,
		cfg_err_cor_n				=> cfg_err_cor_n,
		cfg_err_ur_n				=> cfg_err_ur_n,
		cfg_err_ecrc_n				=> cfg_err_ecrc_n,
		cfg_err_cpl_timeout_n		=> cfg_err_cpl_timeout_n,
		cfg_err_cpl_abort_n			=> cfg_err_cpl_abort_n,
		cfg_err_cpl_unexpect_n		=> cfg_err_cpl_unexpect_n,--
		cfg_err_posted_n			=> cfg_err_posted_n,
		cfg_err_locked_n			=> cfg_err_locked_n,
		cfg_err_tlp_cpl_header		=> cfg_err_tlp_cpl_header,
		cfg_err_cpl_rdy_n			=> cfg_err_cpl_rdy_n,
		cfg_interrupt_n				=> cfg_interrupt_n,
		cfg_interrupt_rdy_n			=> cfg_interrupt_rdy_n,
		cfg_interrupt_assert_n		=> cfg_interrupt_assert_n,
		cfg_interrupt_di			=> cfg_interrupt_di,
		cfg_interrupt_do			=> cfg_interrupt_do,
		cfg_interrupt_mmenable		=> cfg_interrupt_mmenable,
		cfg_interrupt_msienable		=> cfg_interrupt_msienable,
		cfg_pcie_link_state_n		=> cfg_pcie_link_state_n,
		cfg_trn_pending_n			=> cfg_trn_pending_n,
		cfg_bus_number				=> cfg_bus_number,
		cfg_device_number			=> cfg_device_number,
		cfg_function_number			=> cfg_function_number,
		cfg_status					=> cfg_status,
		cfg_command					=> cfg_command,
		cfg_dstatus			   		=> cfg_dstatus,
		cfg_dcommand				=> cfg_dcommand,
		cfg_lstatus					=> cfg_lstatus,
		cfg_lcommand				=> cfg_lcommand,

		int_request 	=> int_request,
		int_ack			=> int_ack,
		int_msgnum 		=> ZERO(4 downto 0),
    	cfg_prmcsr 		=> open,
    	cfg_devcsr 		=> open,
    	cfg_linkcsr 	=> cfg_linkcsr,
    	cfg_msicsr 		=> open,
    	cfg_ltssm 		=> cfg_ltssm,

		slv_dataout		=> slv_dataout,
		slv_bytevalid	=> slv_bytevalid,
		slv_bytecount	=> slv_bytecount,
		slv_dwcount		=> open,
		slv_addr		=> slv_addr,
		slv_bar			=> slv_bar,
		slv_readreq		=> slv_readreq,
		slv_cpladdr		=> slv_cpladdr,
		slv_cplparam	=> slv_cplparam,
		slv_writereq	=> slv_writereq,
		slv_write		=> slv_write,
		slv_lastwrite	=> slv_lastwrite,
		slv_io          => open,
		slv_accept		=> slv_accept,
		slv_abort		=> slv_abort,
		slv_ur          => slv_ur,

		dma_rd			=> dma_rd,
		dma_rdaddr		=> dma_rdaddr,
		dma_rdchannel	=> dma_rdchannel,
		dma_rddata		=> dma_rddata,
		dma_wr			=> dma_wr,
		dma_wraddr		=> open,
		dma_wrchannel	=> dma_wrchannel,
		dma_wrdata		=> dma_wrdata,
		dma_wrbytevalid	=> open,

		dma0_regin		=> dma0_regin,
		dma0_regout		=> dma0_regout,
		dma0_param		=> dma0_param,
		dma0_control	=> dma0_control,
		dma0_status		=> dma0_status,
		dma0_fifocnt	=> dma0_fifocnt
		);

	-------------------------------------------------
	-- Clock & reset
	-------------------------------------------------

    mgtclk : IBUFDS_GTXE1
    port map (
    	I => refclk_p,
    	IB => refclk_n,
    	CEB => ZERO(0),
    	ODIV2 => open,
    	O => refclk);
    
    -- Register trn_reset_n to solve timing problems	
    process (trn_clk,trn_reset_n)
    begin
    	if trn_reset_n='0' then
    		rst <='1';       		
    	elsif rising_edge(trn_clk) then
    		rst <='0';
    	end if;
    end process;
    
    rstn <=not rst; 

    usr_led(7) <=cfg_linkcsr(17);	-- Link trained at Gen2 speed
    usr_led(6) <=cfg_linkcsr(23);	-- Link trained in x8 
    usr_led(5) <=cfg_linkcsr(22);	-- Link trained in x4
    usr_led(4) <=cfg_linkcsr(20);	-- Link trained in x1
          
	-------------------------------------------------
	-- PCIEZ REFERENCE DESIGN
	-------------------------------------------------

	app : pcie_app
	generic map (
		DATAPATH => DATAPATH,
		CORE_FREQ => 250
		)
	port map (
		clk => trn_clk,
	    reset_n => rstn,
--	    srst => ZERO(0),
--	    clk_doubled => ZERO(0),	    
	    int_request => int_request,
      int_ack => int_ack,
	    slv_dataout => slv_dataout,
	    slv_bytevalid => slv_bytevalid,
	    slv_bytecount => slv_bytecount,
	    slv_addr => slv_addr,
	    slv_bar => slv_bar,
	    slv_readreq => slv_readreq,
--	    slv_cpladdr => slv_cpladdr,
--	    slv_cplparam => slv_cplparam,
	    slv_writereq => slv_writereq,
	    slv_write => slv_write,
      slv_lastwrite => slv_lastwrite,
	    slv_accept => slv_accept,
	    slv_abort => slv_abort,
	    slv_ur => slv_ur,
--        flash_wr => open,
--        flash_wrdata => open,
--        flash_rddata => ZERO(63 downto 0),
	    dma_rd => dma_rd,
	    dma_rdaddr => dma_rdaddr,
	    dma_rdchannel => dma_rdchannel,
	    dma_rddata => dma_rddata,
	    dma_wr => dma_wr,
	    dma_wrchannel => dma_wrchannel,
	    dma_wrdata => dma_wrdata,
	    dma0_regin => dma0_regin,
	    dma0_regout => dma0_regout,
	    dma0_param => dma0_param,
	    dma0_control => dma0_control,
	    dma0_status => dma0_status,
	    dma0_fifocnt => dma0_fifocnt
      ,

--	    cfg_linkcsr => cfg_linkcsr,                  
--	    cfg_ltssm => cfg_ltssm,
--	    usr_led => usr_led(3 downto 0)
  CAM0_DATA => CAM0_DATA,
  CAM0_CLK => CAM0_CLK,
  CAM0_RESET_N => CAM0_RESET_N,
  CAM0_LINE_VALID => CAM0_LINE_VALID,
  CAM0_FRAME_VALID => CAM0_FRAME_VALID,
  CAM0_SPI_ENABLE => CAM0_SPI_ENABLE,
  CAM0_SPI_CLK => CAM0_SPI_CLK,
  CAM0_SPI_DATA => CAM0_SPI_DATA
		);
    
  camera : camera_model
  generic map(
    picture_width => picture_width,
	picture_height => picture_height,
	line_overhead => line_overhead,
	frame_overhead => frame_overhead
  )
  port map(
    DATA => CAM0_DATA,
    CLK => CAM0_CLK,
    RESET_N => CAM0_RESET_N,
    LINE_VALID => CAM0_LINE_VALID,
    FRAME_VALID => CAM0_FRAME_VALID,
    SPI_ENABLE => CAM0_SPI_ENABLE,
    SPI_CLK => CAM0_SPI_CLK,
    SPI_DATA => CAM0_SPI_DATA
  );

end top_module_arch;

